@@ -15,8 +15,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 #ifndef __GST_QUARK_H__
@@ -96,7 +96,7 @@ typedef enum _GstQuarkId
   GST_QUARK_MESSAGE_STRUCTURE_CHANGE = 67,
   GST_QUARK_MESSAGE_SEGMENT_START = 68,
   GST_QUARK_MESSAGE_SEGMENT_DONE = 69,
-  GST_QUARK_MESSAGE_DURATION = 70,
+  GST_QUARK_MESSAGE_DURATION_CHANGED = 70,
   GST_QUARK_MESSAGE_ASYNC_DONE = 71,
   GST_QUARK_MESSAGE_REQUEST_STATE = 72,
   GST_QUARK_MESSAGE_STREAM_STATUS = 73,
@@ -188,7 +188,13 @@ typedef enum _GstQuarkId
   GST_QUARK_EVENT_SEGMENT_DONE = 159,
   GST_QUARK_EVENT_STREAM_START = 160,
   GST_QUARK_STREAM_ID = 161,
-  GST_QUARK_MAX = 162
+  GST_QUARK_EVENT_CONTEXT = 162,
+  GST_QUARK_QUERY_CONTEXT = 163,
+  GST_QUARK_MESSAGE_NEED_CONTEXT = 164,
+  GST_QUARK_MESSAGE_HAVE_CONTEXT = 165,
+  GST_QUARK_CONTEXT = 166,
+  GST_QUARK_CONTEXT_TYPES = 167,
+  GST_QUARK_MAX = 168
 } GstQuarkId;
 
 extern GQuark _priv_gst_quark_table[GST_QUARK_MAX];
